package gregapi.block.prefixblock;

import static gregapi.data.CS.SHOW_HIDDEN_MATERIALS;
import static gregapi.data.CS.SHOW_HIDDEN_PREFIXES;
import static gregapi.data.CS.UNCOLORED;
import static gregapi.data.CS.W;
import gregapi.data.LH;
import gregapi.data.TD;
import gregapi.item.IItemGT;
import gregapi.item.IItemUpdatable;
import gregapi.oredict.OreDictMaterial;
import gregapi.util.UT;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.world.World;

/**
 * @author Gregorius Techneticies
 */
public class PrefixBlockItem extends ItemBlock implements IItemUpdatable, IItemGT {
	public final PrefixBlock mBlock;
	
	public PrefixBlockItem(Block aBlock) {
		super(aBlock);
        setMaxDamage(0);
        setHasSubtypes(true);
		mBlock = (PrefixBlock)aBlock;
	}
	
	@Override
    public void getSubItems(Item var1, CreativeTabs aCreativeTab, List aList) {
        if (SHOW_HIDDEN_PREFIXES || !mBlock.mPrefix.contains(TD.Creative.HIDDEN)) for (int i = 0; i < OreDictMaterial.MATERIAL_ARRAY.length; i++) if (mBlock.mPrefix.isGeneratingItem(OreDictMaterial.MATERIAL_ARRAY[i])) if (SHOW_HIDDEN_MATERIALS || !OreDictMaterial.MATERIAL_ARRAY[i].contains(TD.Creative.HIDDEN)) {
			ItemStack tStack = new ItemStack(this, 1, i);
			updateItemStack(tStack);
        	aList.add(tStack);
        }
        if (aList.isEmpty()) UT.Stacks.hide(this);
    }
	
    @Override
    public boolean placeBlockAt(ItemStack aStack, EntityPlayer aPlayer, World aWorld, int aX, int aY, int aZ, int aSide, float hitX, float hitY, float hitZ, int aMeta) {
		if (mBlock.placeBlock(aWorld, aX, aY, aZ, (byte)aSide, (short)aStack.getItemDamage(), aStack.getTagCompound(), true, false)) {
	        if (aWorld.getBlock(aX, aY, aZ) == field_150939_a) {
	            field_150939_a.onBlockPlacedBy(aWorld, aX, aY, aZ, aPlayer, aStack);
	            field_150939_a.onPostBlockPlaced(aWorld, aX, aY, aZ, aStack.getItemDamage());
	        }
			return true;
		}
		UT.Entities.chat(aPlayer, "Cannot place Block in this Environment!");
		return false;
    }
    
	@Override
    public int getColorFromItemStack(ItemStack aStack, int aRenderPass) {
		if (aRenderPass == 0) {
			int aMetaData = aStack.getItemDamage();
			if (UT.Code.exists(aMetaData, OreDictMaterial.MATERIAL_ARRAY)) return UT.Code.getRGBInt(OreDictMaterial.MATERIAL_ARRAY[aMetaData].mRGBa[mBlock.mPrefix.mState]);
		}
		return UNCOLORED;
    }
	
	@Override
	public final String getUnlocalizedName(ItemStack aStack) {
		int aMetaData = aStack.getItemDamage();
		if (aMetaData == W) return mBlock.getUnlocalizedName()+".wildcard";
		if (UT.Code.exists(aMetaData, OreDictMaterial.MATERIAL_ARRAY)) return "oredict." + mBlock.mPrefix.get(OreDictMaterial.MATERIAL_ARRAY[aMetaData]).toString();
		return mBlock.getUnlocalizedName();
	}
	
    @Override
    public void addInformation(ItemStack aStack, EntityPlayer aPlayer, List aList, boolean aF3_H) {
    	super.addInformation(aStack, aPlayer, aList, aF3_H);
    	if (mBlock.mSpawnProof) aList.add(LH.Chat.CYAN + LH.get(LH.TOOLTIP_SPAWNPROOF));
    }
    
	@Override public final String getUnlocalizedName() {return mBlock.getUnlocalizedName();}
	@Override public final boolean hasContainerItem(ItemStack aStack) {return getContainerItem(aStack) != null;}
	@Override public ItemStack getContainerItem(ItemStack aStack) {return null;}
	@Override public void updateItemStack(ItemStack aStack) {int aDamage = aStack.getItemDamage(); if (UT.Code.exists(aDamage, OreDictMaterial.MATERIAL_ARRAY)) {OreDictMaterial aMaterial = OreDictMaterial.MATERIAL_ARRAY[aDamage]; if (aMaterial != aMaterial.mTargetRegistration) aStack.setItemDamage(aMaterial.mTargetRegistration.mID);}}
	@Override public void onCreated(ItemStack aStack, World aWorld, EntityPlayer aPlayer) {updateItemStack(aStack);}
	@Override public boolean isBookEnchantable(ItemStack aStack, ItemStack aBook) {return false;}
	@Override public boolean getIsRepairable(ItemStack aStack, ItemStack aMaterial) {return false;}
	@Override public int getItemEnchantability() {return 0;}
	@Override public int getItemStackLimit(ItemStack aStack) {return mBlock.mPrefix.mDefaultStackSize;}
}