package gregapi.render;

import gregapi.old.interfaces.IColorModulationContainer;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;

/** 
 * @author Gregorius Techneticies
 */
public class BlockTextureSided implements ITexture, IColorModulationContainer {
	private final IIconContainer[] mIconContainers;
	
	/**
	 *  DO NOT MANIPULATE THE VALUES INSIDE THIS ARRAY!!!
	 *  
	 *  Just set this variable to another different Array instead.
	 *  Otherwise some coloured things will get Problems.
	 */
	public short[] mRGBa;
	
	private final boolean mAllowAlpha, mUseMaxBrightness;
	
	public BlockTextureSided(IIconContainer aIcon0, IIconContainer aIcon1, IIconContainer aIcon2, IIconContainer aIcon3, IIconContainer aIcon4, IIconContainer aIcon5, short[] aRGBa, boolean aAllowAlpha, boolean aUseMaxBrightness) {
		if (aRGBa.length != 4) throw new IllegalArgumentException("RGBa doesn't have 4 Values @ BlockTextureSided");
		mIconContainers = new IIconContainer[] {aIcon0, aIcon1, aIcon2, aIcon3, aIcon4, aIcon5};
		mUseMaxBrightness = aUseMaxBrightness;
		mAllowAlpha = aAllowAlpha;
		mRGBa = aRGBa;
	}
	
	public BlockTextureSided(IIconContainer aBottom, IIconContainer aTop, IIconContainer aSides, short[] aRGBa, boolean aAllowAlpha, boolean aUseMaxBrightness) {
		this(aBottom, aTop, aSides, aSides, aSides, aSides, aRGBa, aAllowAlpha, aUseMaxBrightness);
	}
	
	@Override
	public void renderXPos(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness, boolean aChangedBlockBounds) {
		final byte aSide = 5;
		for (int i = 0, j = mIconContainers[aSide].getIconPasses(); i < j; i++)
		ITexture.Util.renderZPos(mIconContainers[aSide].getIcon(0), mIconContainers[aSide].isUsingColorModulation(i)?mRGBa:mIconContainers[aSide].getIconColor(i), mAllowAlpha, mUseMaxBrightness, true, aRenderer, aBlock, aX, aY, aZ, mUseMaxBrightness?240:aBrightness, aChangedBlockBounds);
	}
	
	@Override
	public void renderXNeg(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness, boolean aChangedBlockBounds) {
		final byte aSide = 4;
		for (int i = 0, j = mIconContainers[aSide].getIconPasses(); i < j; i++)
		ITexture.Util.renderZPos(mIconContainers[aSide].getIcon(0), mIconContainers[aSide].isUsingColorModulation(i)?mRGBa:mIconContainers[aSide].getIconColor(i), mAllowAlpha, mUseMaxBrightness, true, aRenderer, aBlock, aX, aY, aZ, mUseMaxBrightness?240:aBrightness, aChangedBlockBounds);
	}
	
	@Override
	public void renderYPos(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness, boolean aChangedBlockBounds) {
		final byte aSide = 1;
		for (int i = 0, j = mIconContainers[aSide].getIconPasses(); i < j; i++)
		ITexture.Util.renderZPos(mIconContainers[aSide].getIcon(0), mIconContainers[aSide].isUsingColorModulation(i)?mRGBa:mIconContainers[aSide].getIconColor(i), mAllowAlpha, mUseMaxBrightness, true, aRenderer, aBlock, aX, aY, aZ, mUseMaxBrightness?240:aBrightness, aChangedBlockBounds);
	}
	
	@Override
	public void renderYNeg(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness, boolean aChangedBlockBounds) {
		final byte aSide = 0;
		for (int i = 0, j = mIconContainers[aSide].getIconPasses(); i < j; i++)
		ITexture.Util.renderZPos(mIconContainers[aSide].getIcon(0), mIconContainers[aSide].isUsingColorModulation(i)?mRGBa:mIconContainers[aSide].getIconColor(i), mAllowAlpha, mUseMaxBrightness, true, aRenderer, aBlock, aX, aY, aZ, mUseMaxBrightness?240:aBrightness, aChangedBlockBounds);
	}
	
	@Override
	public void renderZPos(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness, boolean aChangedBlockBounds) {
		final byte aSide = 3;
		for (int i = 0, j = mIconContainers[aSide].getIconPasses(); i < j; i++)
		ITexture.Util.renderZPos(mIconContainers[aSide].getIcon(0), mIconContainers[aSide].isUsingColorModulation(i)?mRGBa:mIconContainers[aSide].getIconColor(i), mAllowAlpha, mUseMaxBrightness, true, aRenderer, aBlock, aX, aY, aZ, mUseMaxBrightness?240:aBrightness, aChangedBlockBounds);
	}
	
	@Override
	public void renderZNeg(RenderBlocks aRenderer, Block aBlock, int aX, int aY, int aZ, int aBrightness, boolean aChangedBlockBounds) {
		final byte aSide = 2;
		for (int i = 0, j = mIconContainers[aSide].getIconPasses(); i < j; i++)
		ITexture.Util.renderZPos(mIconContainers[aSide].getIcon(0), mIconContainers[aSide].isUsingColorModulation(i)?mRGBa:mIconContainers[aSide].getIconColor(i), mAllowAlpha, mUseMaxBrightness, true, aRenderer, aBlock, aX, aY, aZ, mUseMaxBrightness?240:aBrightness, aChangedBlockBounds);
	}
	
	@Override
	public short[] getRGBA() {
		return mRGBa;
	}
	
	@Override
	public boolean isValidTexture() {
		return mIconContainers != null && mIconContainers[0] != null && mIconContainers[1] != null && mIconContainers[2] != null && mIconContainers[3] != null && mIconContainers[4] != null && mIconContainers[5] != null;
	}
}